@@ -1480,12 +1480,12 @@ panel_profile_delete_toplevel_objects (const char        *toplevel_id,
 
 		if (parent_toplevel_id && !strcmp (toplevel_id, parent_toplevel_id)) {
 			g_free (id);
-			g_free (parent_toplevel_id);
 			continue;
 		}
 
 		new_list = g_slist_prepend (new_list, id);
 
+		g_free (parent_toplevel_id);
 	}
 	g_slist_free (list);
 
@@ -2418,6 +2418,12 @@ panel_profile_get_enable_program_list (void)
 	return get_program_listing_setting ("enable_program_list");
 }
 
+gboolean
+panel_profile_get_enable_autocomplete (void)
+{
+	return get_program_listing_setting ("enable_autocomplete");
+}
+
 void
 panel_profile_set_show_program_list (gboolean show_program_list)
 {
